@@ -51,7 +51,12 @@
 #include "llvm/Support/Path.h"
 #include "llvm/Support/raw_ostream.h"
 
+#if LLVM_VERSION_MAJOR >= 20
+#  include "llvm/Support/VirtualFileSystem.h"
+#endif
+
 #include <cstdlib>
+#include <fstream>
 #include <iostream>
 #include <memory>
 #include <queue>
@@ -289,7 +294,6 @@ protected:
                     "#undef __BFLT16_MIN_EXP__\n"
                     "#undef __BFLT16_MIN__\n"
                     "#undef __BFLT16_NORM_MAX__\n"
-                    "#undef __SIZEOF_FLOAT80__\n"
                     "#undef __STDCPP_BFLOAT16_T__\n"
                     "#undef __STDCPP_FLOAT128_T__\n"
                     "#undef __STDCPP_FLOAT16_T__\n"
@@ -306,6 +310,14 @@ protected:
                     "#define __builtin_va_arg_pack_len() 1\n";
       }
 
+      // Provide __float80 if simulating the actual GNU compiler.
+      if (this->NeedFloat80(this->Opts.Predefines)) {
+        // Clang does not support this builtin.  Approximate it.
+        builtins += "\n"
+                    "typedef long double __castxml__float80;\n"
+                    "#define __float80 __castxml__float80\n";
+      }
+
       // Provide __float128 if simulating the actual GNU compiler.
       if (!this->HaveFloat128(CI) &&
           this->NeedFloat128(this->Opts.Predefines)) {
@@ -532,7 +544,7 @@ protected:
     return this->IsActualGNU(pd);
   }
 
-  bool NeedFloat128(std::string const& pd) const
+  bool NeedFloat80(std::string const& pd) const
   {
     return (this->IsActualGNU(pd) &&
             (pd.find("#define __i386__ ") != pd.npos ||
@@ -540,6 +552,11 @@ protected:
              pd.find("#define __ia64__ ") != pd.npos));
   }
 
+  bool NeedFloat128(std::string const& pd) const
+  {
+    return this->NeedFloat80(pd);
+  }
+
   bool HaveFloat128(clang::CompilerInstance const& CI) const
   {
 #if LLVM_VERSION_MAJOR > 3 || LLVM_VERSION_MAJOR == 3 && LLVM_VERSION_MINOR > 8
@@ -552,10 +569,37 @@ protected:
 
   bool Need_Float(std::string const& pd) const
   {
-    // gcc >= 7  provides _Float## types in C.
-    // g++ >= 13 provides _Float## types in C++.
-    return (this->IsActualGNU(pd) &&
-            this->GetGNUMajorVersion(pd) >= (this->IsCPlusPlus(pd) ? 13 : 7));
+    if (this->IsActualGNU(pd)) {
+      // gcc >= 7  provides _Float## types in C.
+      if (!this->IsCPlusPlus(pd)) {
+        return this->GetGNUMajorVersion(pd) >= 7;
+      }
+      // g++ >= 13 provides _Float## types in C++.
+      if (this->GetGNUMajorVersion(pd) < 13) {
+        return false;
+      }
+      // glibc 2.27 added bits/floatn-common.h to define _Float## types when
+      // the compiler does not, but only knew about GCC 7+ _Float## types in C.
+      // glibc 2.37 updated the conditions for GCC 13+ _Float## types in C++.
+      for (Options::Include const& i : this->Opts.Includes) {
+        if (i.Framework) {
+          continue;
+        }
+        if (std::ifstream f{ i.Directory + "/bits/floatn-common.h" }) {
+          std::string h{ std::istreambuf_iterator<char>(f),
+                         std::istreambuf_iterator<char>() };
+          // If the header contains the pre-2.37 condition then
+          // let it handle defining the _Float## types.
+          if (h.find("if !__GNUC_PREREQ (7, 0) || defined __cplusplus") !=
+              std::string::npos) {
+            return false;
+          }
+          break;
+        }
+      }
+      return true;
+    }
+    return false;
   }
 
 #if LLVM_VERSION_MAJOR < 6
@@ -698,7 +742,11 @@ static bool isObjC(clang::CompilerInstance* CI)
 static bool runClangCI(clang::CompilerInstance* CI, Options const& opts)
 {
   // Create a diagnostics engine for this compiler instance.
-  CI->createDiagnostics();
+  CI->createDiagnostics(
+#if LLVM_VERSION_MAJOR >= 20
+    *llvm::vfs::getRealFileSystem()
+#endif
+  );
   if (!CI->hasDiagnostics()) {
     return false;
   }
@@ -769,7 +817,11 @@ runClangCreateDiagnostics(const char* const* argBeg, const char* const* argEnd)
     new clang::TextDiagnosticPrinter(llvm::errs(), &*diagOpts);
   llvm::IntrusiveRefCntPtr<clang::DiagnosticsEngine> diags(
     new clang::DiagnosticsEngine(diagID, &*diagOpts, diagClient));
-  clang::ProcessWarningOptions(*diags, *diagOpts, /*ReportDiags=*/false);
+  clang::ProcessWarningOptions(*diags, *diagOpts,
+#if LLVM_VERSION_MAJOR >= 20
+                               *llvm::vfs::getRealFileSystem(),
+#endif
+                               /*ReportDiags=*/false);
   return diags;
 }
 
@@ -895,6 +947,9 @@ int runClang(const char* const* argBeg, const char* const* argEnd,
 
     // Configure language options to match given compiler.
     std::string const& pd = opts.Predefines;
+    if (pd.find("#define __cpp_sized_deallocation ") != std::string::npos) {
+      args.emplace_back("-fsized-deallocation");
+    }
     if (pd.find("#define _MSC_EXTENSIONS ") != pd.npos) {
       args.push_back("-fms-extensions");
     }
